<?php
/**
 * @package     DOCman
 * @copyright   Copyright (C) 2011 Timble CVBA. (http://www.timble.net)
 * @license     GNU GPLv3 <http://www.gnu.org/licenses/gpl.html>
 * @link        http://www.joomlatools.com
 */

/**
 * Limitable Controller Behavior
 *
 * @author  Mati Kochen <https://github.com/kochen>
 * @package Joomlatools\Plugin\DOCman
 */
class PlgDocmanLimitBehaviorLimitable extends KControllerBehaviorAbstract
{
    /**
     * The number of downloads allowed for a document
     */
    protected $_allowed_downloads;

    /**
     * The number of days to limit downloads.
     */
    protected $_days_interval;

    /**
     * Tells if the download limitation should be applied globally or in a per document basis.
     *
     * @var bool
     */
    protected $_limit_documents;

    /**
     * An array containing usergroups for which the limitation applies.
     *
     * @var array
     */
    protected $_usergroups;

    /**
     * Constructor.
     *
     * @param   KObjectConfig $config Configuration options
     */
    public function __construct(KObjectConfig $config)
    {
        parent::__construct($config);

        $this->_allowed_downloads = (int) $config->allowed_downloads;
        $this->_days_interval     = (int) $config->days_interval;
        $this->_limit_documents      = (bool) $config->limit_documents;
        $this->_usergroups         = KObjectConfig::unbox($config->usergroups);
    }

    protected function _initialize(KObjectConfig $config)
    {
        $config->append(array(
            'priority'          => KCommandHandlerInterface::PRIORITY_HIGH,
            'allowed_downloads' => 0,
            'days_interval'     => 0,
            'limit_documents'   => false,
            'usergroups'        => array(),
        ));

        parent::_initialize($config);
    }

    /**
     * Prevents download if the limit is reached
     *
     * @param KControllerContextInterface $context
     */
    protected function _beforeRender(KControllerContextInterface $context)
    {
        $user = $context->getUser();

        if ($this->_usergroups) {
            $whitelisted = !array_intersect($this->_usergroups, $user->getGroups());
        } else {
            $whitelisted = false;
        }

        // Whitelist users that have DOCman backend access.
        if (!$whitelisted && $user->authorise('core.manage', 'com_docman') === true) {
            $whitelisted = true;
        }

        if (!$whitelisted && $this->isDownloadLimitReached($context))
        {
            $translator = $this->getObject('translator');

            $message = $translator->translate('This document cannot be downloaded because you have reached the allowed amount of downloads.');
            $url = $this->getReferrer($context);

            $this->getResponse()->setRedirect($url, $message, 'error')->send();
        }
    }

    /**
     * Activities model getter.
     *
     * @return KModelInterface|null The model, null if the model wasn't found.
     */
    protected function _getActivitiesModel()
    {
        $model = null;

        $identifier = $this->getIdentifier('com://admin/logman.model.activities');

        if ($this->getObject('manager')->getClass($identifier, false)) {
            $model = $this->getObject($identifier);
        }

        return $model;
    }

    /**
     * Returns true if limit was reached
     *
     * @param  KControllerContextInterface $context
     * @return bool
     */
    protected function isDownloadLimitReached(KControllerContextInterface $context)
    {
        $result = false;

        if ($this->_allowed_downloads)
        {
            $document = $this->getModel()->fetch();

            if (!$document->isNew())
            {
                $end_date  = $this->getObject('lib:date',array('date' => date('Y-m-d')));

                $query = $this->getObject('lib:database.query.select')
                              ->table(array('tbl' => 'logman_activities'))
                              ->columns('COUNT(*)')
                              ->where('DATE(tbl.created_on) <= :end')
                              ->where('tbl.package = :package')
                              ->where('tbl.name = :name')
                              ->where('tbl.action = :action')
                              ->bind(array(
                                  'package'   => 'docman',
                                  'name'      => 'document',
                                  'action'    => 'download',
                                  'end'       => $end_date->format('Y-m-d'),
                              ));

                if ($this->_days_interval)
                {
                    $query->where('DATE(tbl.created_on) >= :range_end')->bind(array(
                        'range_end' => $end_date->modify(sprintf('-%d days', $this->_days_interval))
                                                ->format('Y-m-d')
                    ));
                }

                if ($this->_limit_documents) {
                    $query->where('row = :row')->bind(array('row' => $document->id)); // Limit per document.
                }

                $user = $context->getUser();

                if (!$user->isAuthentic()) {
                    $query->where('tbl.ip = :ip')
                          ->where('tbl.created_by = :user')
                          ->bind(array(
                              'ip'   => $context->getRequest()->getAddress(),
                              'user' => 0
                          )); // Filter by IP address.
                } else {
                    $query->where('tbl.created_by = :user')->bind(array('user' => $user->getId()));
                }

                $db = JFactory::getDbo();

                $db->setQuery($query->toString());

                if ($downloads = $db->loadResult()) {
                    $result = $downloads >= $this->_allowed_downloads;
                }
            }
        }

        return $result;
    }
}